import { introspectionFromSchema } from 'graphql';
import _ from 'lodash-es';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var scalarLocations = function scalarLocations(introspectionQuery, scalars) {
  var wantedScalar = function wantedScalar(n) {
    return scalars.indexOf(n) > -1;
  };

  var inputTypes = introspectionQuery.__schema.types.filter(function (_ref) {
    var kind = _ref.kind;
    return kind === "INPUT_OBJECT";
  }).reduce(function (a, x) {
    a[x.name] = x;
    return a;
  }, {});

  var outputTypes = introspectionQuery.__schema.types.filter(function (_ref2) {
    var kind = _ref2.kind;
    return kind === "OBJECT";
  }).reduce(function (a, x) {
    a[x.name] = x;
    return a;
  }, {});

  var interfaceTypes = introspectionQuery.__schema.types.filter(function (_ref3) {
    var kind = _ref3.kind;
    return kind === "INTERFACE";
  }).reduce(function (a, x) {
    a[x.name] = x;
    return a;
  }, {});
  /* Input Object types */


  var inputObjectFieldTypes = {}; // _.fromPairs(scalars.map(type => [type, type]));

  for (var name in inputTypes) {
    var typeInfo = inputTypes[name];
    var typeMap = {};

    for (var _iterator = _createForOfIteratorHelperLoose(typeInfo.inputFields), _step; !(_step = _iterator()).done;) {
      var field = _step.value;
      var fieldType = unpackInputType(field.type);
      if (fieldType === undefined) continue; // ENUMs, others?

      if (fieldType.kind === "SCALAR" && wantedScalar(fieldType.name)) {
        typeMap[field.name] = fieldType.name;
      } else if (fieldType.kind === "INPUT_OBJECT") {
        typeMap[field.name] = fieldType.name;
      }
    }

    if (!_.isEmpty(typeMap)) inputObjectFieldTypes[name] = typeMap;
  }
  /* OUTPUT: split between objects and operation type */


  function getObjectFieldTypes(typeInfo) {
    var typeMap = {};

    for (var _iterator2 = _createForOfIteratorHelperLoose(typeInfo.fields), _step2; !(_step2 = _iterator2()).done;) {
      var _field = _step2.value;

      var _fieldType = unpackOutputType(_field.type);

      if (_fieldType === undefined) continue; // ENUMs, others?

      if (_fieldType.kind === "OBJECT" && _fieldType.name.startsWith("__")) continue;

      if (_fieldType.kind === "SCALAR" && wantedScalar(_fieldType.name)) {
        typeMap[_field.name] = _fieldType.name;
      } else if (_fieldType.kind === "OBJECT") {
        typeMap[_field.name] = _fieldType.name;
      } else if (_fieldType.kind === "INTERFACE") {
        var iface = interfaceTypes[_fieldType.name];
        typeMap[_field.name] = iface.possibleTypes.map(function (t) {
          return t.name;
        });
      }
    }

    return typeMap;
  }

  var outputObjectFieldTypes = {};
  var operationMap = {};

  for (var _name in outputTypes) {
    if (_name === "RootMutationType" || _name === "RootQueryType") {
      var rootType = outputTypes[_name];
      Object.assign(operationMap, getObjectFieldTypes(rootType));
    } else {
      var objectType = outputTypes[_name];
      if (_name.startsWith("__")) continue; // __Schema, __Type, and other metadatas

      var fields = getObjectFieldTypes(objectType);
      if (!_.isEmpty(fields)) outputObjectFieldTypes[_name] = fields;
    }
  }

  return {
    inputObjectFieldTypes: inputObjectFieldTypes,
    outputObjectFieldTypes: outputObjectFieldTypes,
    operationMap: operationMap
  };
};
/*
const makeTree = (paths : FieldPath[], includeTypeName : boolean) => {
  const scalarTree : Node = {}

  const nest = (map: Node, path: string[], idx: number, fieldType: string) => {
    if (idx >= path.length - 1) {
      // last element
      if (path[idx] in map) throw new Error(`Path ${path} already exists in ${map}`)
      map[path[idx]] = fieldType;
    } else {
      const next = map[path[idx]] || {};
      if (typeof next === 'string') throw new Error(`Path ${path} overlaps with another path in existing map ${map}`)
      map[path[idx]] = next;
      nest(next, path, idx + 1, fieldType);
    }
  }

  for (const fp of paths) {
    const p = includeTypeName ? [fp.type].concat(fp.path) : fp.path;
    nest(scalarTree, p, 0, fp.fieldType);
  }
  return scalarTree;
}
*/


var unpackInputType = function unpackInputType(type) {
  if (type.kind === "SCALAR" || type.kind === "INPUT_OBJECT") return type;
  if (type.kind === "LIST" || type.kind === "NON_NULL") return unpackInputType(type.ofType);
  return;
};

var unpackOutputType = function unpackOutputType(type) {
  if (type.kind === "SCALAR" || type.kind === "OBJECT" || type.kind === "INTERFACE") return type;
  if (type.kind === "LIST" || type.kind === "NON_NULL") return unpackOutputType(type.ofType);
  return;
};

var plugin = function plugin(schema, _documents, config) {
  if (!config.scalars) throw new Error("graphql-codegen-scalar-locations needs config key 'scalars' with list of scalar names");
  var intro = introspectionFromSchema(schema);
  var locations = scalarLocations(intro, config.scalars);
  return "\nexport type ObjectFieldTypes = {\n    [key: string]: { [key: string]: string | string[] }\n};\n\nexport type OpTypes = {\n    [key: string]: string | string[]\n};\n\nexport type ScalarLocations = {\n scalars: string[],\n inputObjectFieldTypes: ObjectFieldTypes;\n outputObjectFieldTypes: ObjectFieldTypes;\n operationMap: OpTypes;\n};\n\nexport const scalarLocations : ScalarLocations = " + JSON.stringify(_extends({}, locations, {
    scalars: config.scalars
  }), null, 2) + ";\n";
};

export { plugin };
//# sourceMappingURL=codegen-graphql-scalar-locations.esm.js.map
